#pragma once

#include <IsoEnginePCH.h>

enum class LogLevel {
    TRACE_l,
    DEBUG_l,
    INFO_l,
    WARN_l,
    ERROR_l,
    FATAL_l
};

class Logger {
public:
    static Logger& GetCoreLogger();
    static Logger& GetAppLogger();

    void Log(LogLevel level, const std::string& message);

    template<typename... Args>
    void LogFormat(LogLevel level, const std::string& format, Args&&... args) {
        if (level >= logLevel) {
            std::string formattedMessage = std::vformat(format, std::make_format_args(std::forward<Args>(args)...));
            LogInternal(level, formattedMessage);
        }
    }

    void AddOutputStream(std::ostream* stream);
    void SetLogLevel(LogLevel level);

    template<typename... Args>
    static void FatalHandler(const std::string& format, Args... args);

private:
    Logger();
    void LogInternal(LogLevel level, const std::string& message);
    std::string GetCurrentTime();
    std::string LevelToString(LogLevel level);

    std::vector<std::ostream*> outputStreams;
    LogLevel logLevel = LogLevel::TRACE_l;
};

#ifdef ENABLE_LOGGING

#define LOG_TRACE(...) Logger::GetAppLogger().LogFormat(LogLevel::TRACE_l, __VA_ARGS__)
#define LOG_DEBUG(...) Logger::GetAppLogger().LogFormat(LogLevel::DEBUG_l, __VA_ARGS__)
#define LOG_INFO(...)  Logger::GetAppLogger().LogFormat(LogLevel::INFO_l, __VA_ARGS__)
#define LOG_WARN(...)  Logger::GetAppLogger().LogFormat(LogLevel::WARN_l, __VA_ARGS__)
#define LOG_ERROR(...) Logger::GetAppLogger().LogFormat(LogLevel::ERROR_l, __VA_ARGS__)
#define LOG_FATAL(...) Logger::FatalHandler(__VA_ARGS__)

#define CORE_LOG_TRACE(...) Logger::GetCoreLogger().LogFormat(LogLevel::TRACE_l, __VA_ARGS__)
#define CORE_LOG_DEBUG(...) Logger::GetCoreLogger().LogFormat(LogLevel::DEBUG_l, __VA_ARGS__)
#define CORE_LOG_INFO(...)  Logger::GetCoreLogger().LogFormat(LogLevel::INFO_l, __VA_ARGS__)
#define CORE_LOG_WARN(...)  Logger::GetCoreLogger().LogFormat(LogLevel::WARN_l, __VA_ARGS__)
#define CORE_LOG_ERROR(...) Logger::GetCoreLogger().LogFormat(LogLevel::ERROR_l, __VA_ARGS__)
#define CORE_LOG_FATAL(...) Logger::FatalHandler(__VA_ARGS__)

#else

#define LOG_TRACE(...)
#define LOG_DEBUG(...)
#define LOG_INFO(...)
#define LOG_WARN(...)
#define LOG_ERROR(...)
#define LOG_FATAL(...) Logger::FatalHandler(__VA_ARGS__)

#define CORE_LOG_TRACE(...)
#define CORE_LOG_DEBUG(...)
#define CORE_LOG_INFO(...)
#define CORE_LOG_WARN(...)
#define CORE_LOG_ERROR(...)
#define CORE_LOG_FATAL(...) Logger::FatalHandler(__VA_ARGS__)

#endif
